/*
 * #%L
 * EUGene :: EUGene
 * 
 * $Id: ObjectModel.java 1012 2010-11-28 11:24:27Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.3.3/eugene/src/main/java/org/nuiton/eugene/models/object/ObjectModel.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */


package org.nuiton.eugene.models.object;

import java.util.Collection;
import java.util.List;

import org.nuiton.eugene.models.Model;

/**
 * Abstraction for the root node of object model trees.
 * This an entry point for browsing a model tree. This object offers
 * as well several facilities for a direct access to some of the object model elements. 
 * 
 * Created: 14 janv. 2004
 *
 * @author Cedric Pineau <pineau@codelutin.com>
 * Copyright Code Lutin
 * @version $Revision: 1012 $
 *
 * Last update : $Date: 2010-11-28 12:24:27 +0100 (dim, 28 nov 2010) $
 * by : */
public interface ObjectModel extends Model {
    /**
     * Plexus role-hint
     */
    String NAME = "objectmodel";
    
    /**
    * Returns all classifiers defined in this model. (Except innerClasses)
    * @see ObjectModelClassifier
    * 
    * @return a Collection containing all ObjectModelClassifier for this model.
    */
    Collection<ObjectModelClassifier> getClassifiers();

    /**
    * Returns the classifier corresponding to the given qualified name, or null if the model contains no classifier for this qualified name.
    * @param qualifiedClassifierName - the qualified name of the classifier to retrieve.
    * 
    * @return the ObjectModelClassifier of the found classifier, or null if the model contains no classifier for this qualified name.
    */
    ObjectModelClassifier getClassifier(String qualifiedClassifierName);

    /**
    * Returns all classes defined in this model. (Except innerClasses)
    * @see ObjectModelClass
    * 
    * @return a Collection containing all ObjectModelClass for this model.
    */
    Collection<ObjectModelClass> getClasses();

    /**
    * Returns the class corresponding to the given qualified name, or null if the model contains no class for this qualified name.
    * @param qualifiedClassName - the qualified name of the class to retrieve.
    * 
    * @return the ObjectModelClass of the found class, or null if the model contains no class for this qualified name.
    */
    ObjectModelClass getClass(String qualifiedClassName);


    /**
    * Indicates whether the model contains the class associated to the given className
    * @param qualifiedClassName - the qualified name of the class to retrieve.
    * 
    * @return true if the class has been found.
    */
    boolean hasClass(String qualifiedClassName);

    /**
    * Returns all interfaces defined in this model.
    * @see ObjectModelInterface
    * 
    * @return a Collection containing all ObjectModelInterface for this model.
    */
    Collection<ObjectModelInterface> getInterfaces();

    /**
    * Returns the interface corresponding to the given qualified name, or null if the model contains no interface for this qualified name.
    * @param qualifiedInterfaceName the qualified name of the interface to retrieve.
    * 
    * @return the ObjectModelInterface of the found interface, or null if the model contains no interface for this qualified name.
    */
    ObjectModelInterface getInterface(String qualifiedInterfaceName);


    /**
     * Returns all enumerations defined in this model.
     * @see ObjectModelEnumeration
     *
     * @return a Collection containing all ObjectModelEnumeration for this model.
     */
    Collection<ObjectModelEnumeration> getEnumerations();

    /**
     * Return the enumeration corresponding to the given qualified name
     * @param qualifiedEnumerationName the fully qualified name of the enumeration to retrieve.
     * @return the ObjectModelEnumeration of the found enumeration or null if the model contains no enumeration for this qualified name.
     */
    ObjectModelEnumeration getEnumeration(String qualifiedEnumerationName);

    /**
    * Returns all comments not lied to a particular model element
    * 
    * @return a List containing all comments for this model as Strings.
    */
    List<String> getComments();

    /**
     * Get the extension associated to the reference (unique)
     * @param <O> object type returned
     * @param reference unique corresponding to the extension to get
     * @param extensionClass class of the extension
     * @return the object value for the extension
     * @throws ClassCastException when extensionClass is not valid
     * @throws IllegalArgumentException for non existing extension with reference
     */
     <O> O getExtension(String reference, Class<O> extensionClass)
            throws ClassCastException, IllegalArgumentException;

} //ObjectModel
